import AutenticacaoServices from "../../services/loginServices.js";
import Jwt from "jsonwebtoken";
import UsuarioRepository from "../../repository/usuarioRepository.js";
import Hash from "../../utils/hash.js";
import LoginSchema from "../../schema/loginSchema.js";

// Mock das dependências
jest.mock('../../repository/usuarioRepository.js');
jest.mock('../../utils/hash.js');
jest.mock('jsonwebtoken');
jest.mock('../../schema/loginSchema.js');

describe('AutenticacaoServices', () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    it('deve gerar um token de autenticação quando o login for bem-sucedido', async () => {
        // Dados de entrada
        const data = { siape: '12345', senha: 'senhaValida' };

        // Mock do LoginSchema
        LoginSchema.loginSchema.parse.mockReturnValue(data);

        // Mock do UsuarioRepository
        const usuarioMock = [{ id: 1, siape: '12345', ativo: true, senha: 'hashedPassword' }];
        UsuarioRepository.findAll.mockResolvedValue({ data: usuarioMock }); // Retorno com a chave 'data'

        // Mock do Hash (comparação de senha)
        Hash.compararSenha.mockResolvedValue(true);

        // Mock do Jwt
        Jwt.sign.mockReturnValue('tokenGerado');

        // Chamar o método de login
        const token = await AutenticacaoServices.login(data);

        // Verificar se o método Jwt.sign foi chamado corretamente
        expect(Jwt.sign).toHaveBeenCalledWith(
             usuarioMock[0] ,
            process.env.JWT_SECRET,
            { expiresIn: '30d' }
        );

        // Verificar se o token gerado foi o esperado
        expect(token).toBe('tokenGerado');
    });

    it('deve lançar erro quando o usuário não for encontrado', async () => {
        const data = { siape: '1234', senha: 'senhaValida' };

        // Mock do LoginSchema
        LoginSchema.loginSchema.parse.mockReturnValue(data);

        // Mock do UsuarioRepository (usuário não encontrado)
        UsuarioRepository.findAll.mockResolvedValue({ data: [] }); // Retorno com a chave 'data' e array vazio

        // Esperar que o erro seja lançado
        await expect(AutenticacaoServices.login(data)).rejects.toEqual({
            message: 'Usuario não encontrado!',
            code: 400,
        });
    });

    it('deve lançar erro quando o usuário estiver inativo', async () => {
        const data = { siape: '12345', senha: 'senhaValida' };

        // Mock do LoginSchema
        LoginSchema.loginSchema.parse.mockReturnValue(data);

        // Mock do UsuarioRepository (usuário inativo)
        const usuarioMock = [{ id: 1, siape: '12345', ativo: false, senha: 'hashedPassword' }];
        UsuarioRepository.findAll.mockResolvedValue({ data: usuarioMock }); // Retorno com a chave 'data'

        // Esperar que o erro seja lançado
        await expect(AutenticacaoServices.login(data)).rejects.toEqual({
            message: 'Usuario inativo!',
            code: 400,
        });
    });

    it('deve lançar erro quando a senha for inválida', async () => {
        const data = { siape: '12345', senha: 'senhaErrada' };

        // Mock do LoginSchema
        LoginSchema.loginSchema.parse.mockReturnValue(data);

        // Mock do UsuarioRepository (usuário encontrado)
        const usuarioMock = [{ id: 1, siape: '12345', ativo: true, senha: 'hashedPassword' }];
        UsuarioRepository.findAll.mockResolvedValue({ data: usuarioMock }); // Retorno com a chave 'data'

        // Mock do Hash (senha inválida)
        Hash.compararSenha.mockResolvedValue(false);

        // Esperar que o erro seja lançado
        await expect(AutenticacaoServices.login(data)).rejects.toEqual({
            message: 'Senha invalida!',
            code: 400,
        });
    });
});
