'use client';


import {
  Card,
  CardContent,
  CardDescription,
  CardFooter,
  CardHeader,
  CardTitle,
} from '@/components/ui/card';
import { FormProvider, useForm, useWatch } from 'react-hook-form';
import InputSelect from './Select';
import { FormControl, FormField, FormItem, FormLabel, FormMessage } from './ui/form';
import { Button } from '@/components/ui/button';
import { CalendarIcon } from 'lucide-react';
import { Calendar } from '@/components/ui/calendar';
import { cn } from '@/lib/utils';
import { format } from 'date-fns';
import { ptBR } from 'date-fns/locale';

import { Popover, PopoverContent, PopoverTrigger } from '@/components/ui/popover';
import { useMutation, useQuery } from '@tanstack/react-query';
import fetchApi from '@/services/fetApi';
import { useEffect, useState } from 'react';

export default function ModalEnsalamento() {
  const form = useForm({
    defaultValues: {
      bloco: '',
      sala: '',
      turma: '',
      docente: '',
      horarioInicio: '',
      horarioFim: '',
      descricao: '',
      dataInicio: '',
      dataFim: '',
    },
  });

  // Watch para monitorar as mudanças no formulário
  const formValues = useWatch({
    control: form.control,
  });

  // Atualiza o body sempre que os valores do formulário mudarem
  const [body, setBody] = useState({
    ensalamento: [],
    dataInicio: null,
    dataFim: null,
  });

  useEffect(() => {
    setBody({
      ensalamento: [
        {
          id_sala: Number(formValues.sala),
          id_usuario: Number(formValues.docente),
          turma: formValues.turma,
          hora_inicio: `2024-01-01T${formValues.horarioInicio}:00.000z`,
          hora_fim: `2024-01-01T${formValues.horarioFim}:00.000z`,	
          descricao: formValues.descricao,
          id_reservante: 10,
        },
      ],
      dataInicio: formValues.dataInicio ? new Date(formValues.dataInicio).toISOString() : null,
      dataFim: formValues.dataFim ? new Date(formValues.dataFim).toISOString() : null,
    });
    console.log('body', body);
  }, [formValues]);

  const blocoSelecionado = form.watch('bloco');

  // Consultas para buscar dados de Bloco, Sala e Docente
  const { isPending, data } = useQuery({
    queryKey: ['getBlocoModal'],
    queryFn: async () => {
      const response = await fetchApi('http://localhost:7002/sala?limit=97', 'GET');
      return response.error ? [] : [...new Map(response.data.map((item) => [item.bloco, item])).values()];
    },
  });

  const { data: dataSala } = useQuery({
    queryKey: ['getSalaModal', blocoSelecionado],
    queryFn: async () => {
      const response = await fetchApi(`http://localhost:7002/sala?limit=97&bloco=${blocoSelecionado}`, 'GET');
      return response.error ? [] : response.data;
    },
    enabled: !!blocoSelecionado,
  });

  const { data: dataDocente } = useQuery({
    queryKey: ['getDocenteModal'],
    queryFn: async () => {
      const response = await fetchApi(`http://localhost:7002/usuario?dape=false&limit=97`, 'GET');
      return response.error ? [] : response.data;
    },
  });

  const listaHorarios = [
    '07:10', '08:00', '08:50', '09:55', '10:45', '11:35',
    '13:10', '14:00', '14:50', '15:55', '16:45', '17:35',
    '19:00', '19:50', '20:55', '21:45',
  ];

  const [modal, setModal] = useState(false);

  // Chamada da API usando useMutation
  const { mutate, isLoading: isLoadingEnsalamento, error: errorEnsalamento } = useMutation({
    mutationFn: async () => {
      const response = await fetchApi('http://localhost:7002/ensalamento', 'POST', body);
      if (response.error) {
        throw new Error(response.message);
      }
      return response.data;
    },
    onSuccess: (data) => {
      console.log('Ensalamento realizado com sucesso:', data);
      alert('Ensalamento realizado com sucesso!');
      setModal(false); // Fecha o modal após sucesso
    },
    onError: (error) => {
      console.error('Erro ao realizar ensalamento:', error);
      alert(`Erro ao realizar ensalamento: ${error.message}`);
    },
  });

  return (
    <div className="">
      {modal && (
        <div className="">
          <Card className="w-[460px] h-[700px] p-4 absolute top-9 left-56 shadow-lg border-1 border-black">
            <FormProvider {...form}>
              <form onSubmit={(e) => {
                e.preventDefault();
                mutate(); // Executa a chamada da API
              }}>
                <InputSelect
                  name="bloco"
                  titulo="Bloco"
                  placeholder="Selecione um Bloco"
                  options={isPending ? [] : data.map((sala) => ({ id: sala.bloco, nome: sala.bloco }))}
                  control={form.control}
                  w="w-[80px]"
                />
                <InputSelect
                  name="sala"
                  titulo="Sala"
                  placeholder="Selecione uma Sala"
                  options={dataSala ? dataSala.map((sala) => ({ id: sala.id, nome: sala.nome })) : []}
                  control={form.control}
                  w="w-[200px]"
                />
                <FormLabel>Turma</FormLabel>
                <FormControl>
                  <input
                    {...form.register('turma')}
                    type="text"
                    className="mt-1 block w-[60%] px-3 py-2 border rounded-md"
                  />
                </FormControl>
                <div className='flex gap-1'>
                  <InputSelect
                    name="horarioInicio"
                    titulo="Horário de Início"
                    placeholder="Selecione um horário"
                    options={listaHorarios.map((horario) => ({ id: horario, nome: horario }))}
                    control={form.control}
                    w="w-[200px]"
                  />
                  <InputSelect
                    name="horarioFim"
                    titulo="Horário de término"
                    placeholder="Selecione um horário"
                    options={listaHorarios.map((horario) => ({ id: horario, nome: horario }))}
                    control={form.control}
                    w="w-[200px]"
                  />
                </div>
                <InputSelect
                  name="docente"
                  titulo="Docente"
                  placeholder="Selecione um Docente"
                  options={dataDocente ? dataDocente.map((doc) => ({ id: doc.id, nome: doc.nome })) : []}
                  control={form.control}
                  w="w-[200px]"
                />
                <FormLabel className={''}>Descrição</FormLabel>
                <FormControl>
                  <input
                    {...form.register('descricao')}
                    type="text"
                    className="mt-1 block w-[60%] px-3 py-2 border rounded-md"
                  />
                </FormControl>
                 {/* Calendário */}

                 <FormField
                  control={form.control}
                  name="dataInicio"
                  render={({ field }) => (
                    <FormItem className="flex flex-col">
                      <FormLabel className='mb-[9px]'>Data de Início</FormLabel>
                      <Popover>
                        <PopoverTrigger asChild>
                          <FormControl>
                            <Button
                              variant={"outline"}
                              className={cn(
                                "w-[240px] pl-3 text-left font-normal",
                                !field.value && "text-muted-foreground"
                              )}
                            >
                              {field.value ? (
                                format(field.value, "dd 'de' MMMM 'de' yyyy", { locale: ptBR })
                              ) : (
                                <span>Selecione uma data</span>
                              )}
                              <CalendarIcon className="ml-auto h-4 w-4 opacity-50" />
                            </Button>
                          </FormControl>
                        </PopoverTrigger>
                        <PopoverContent className="w-auto p-0" align="start">
                          <Calendar
                            mode="single"
                            selected={field.value}
                            onSelect={(date) => {
                              field.onChange(date); // Atualiza o valor do campo com a data
                
                            }}
                          />
                        </PopoverContent>
                      </Popover>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                {/* Calendário */}

                <FormField
                  control={form.control}
                  name="dataFim"
                  render={({ field }) => (
                    <FormItem className="flex flex-col">
                      <FormLabel className='mb-[9px]'>Data de término</FormLabel>
                      <Popover>
                        <PopoverTrigger asChild>
                          <FormControl>
                            <Button
                              variant={"outline"}
                              className={cn(
                                "w-[240px] pl-3 text-left font-normal",
                                !field.value && "text-muted-foreground"
                              )}
                            >
                              {field.value ? (
                                format(field.value, "dd 'de' MMMM 'de' yyyy", { locale: ptBR })
                              ) : (
                                <span>Selecione uma data</span>
                              )}
                              <CalendarIcon className="ml-auto h-4 w-4 opacity-50" />
                            </Button>
                          </FormControl>
                        </PopoverTrigger>
                        <PopoverContent className="w-auto p-0" align="start">
                          <Calendar
                            mode="single"
                            selected={field.value}
                            onSelect={(date) => {
                              field.onChange(date); // Atualiza o valor do campo com a data
                            }}
                          />
                        </PopoverContent>
                      </Popover>
                      <FormMessage />
                    </FormItem>
                  )}
                />

                <Button className={'mt-2 bg-primary'} type="submit" disabled={isLoadingEnsalamento}>
                  {isLoadingEnsalamento ? 'Enviando...' : 'Realizar Ensalamento'}
                </Button>
                <Button className={'ml-2'} onClick={() => setModal(false)} disabled={isLoadingEnsalamento}>
                  Cancelar
                </Button>
                {errorEnsalamento && <p className="text-red-500 mt-2">{errorEnsalamento.message}</p>}
              </form>
            </FormProvider>
          </Card>
        </div>
      )}
      <Button className='bg-primary' onClick={() => setModal(true)}> Ralizar ensalamento</Button>
    </div>
  );
}